/* -*- C++ -*-
 * This file is part of the pSOS simulation model for CarbonKernel.
 * Copyright (C) 2001  Philippe Gerum.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description: Interface to the pSOS+m memory managers.
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#ifndef _pSOSMemory_h
#define _pSOSMemory_h

#if defined(__GNUG__) && !defined(__OBSCAN__)
#pragma interface
#endif // __GNUG__ && !__OBSCAN__

#include "CkMemory.h"
#include "pSOSSynch.h"

#ifdef IMPLEMENTING_SO
#undef IMPLEMENTING_SO
#endif // IMPLEMENTING_SO
#define IMPLEMENTING_SO IMPLEMENT_PSOS
#include "sodecl.h"

class pSOSNode;

class pSOSPartition : public CkPartition {

private:

    int deleteOverride;

protected:

    static const char *stateArray[4];

public:

    pSOSPartition(const char *name,
		  CkExtId uname,
		  void *startAddr,
		  ckulw_t psize,
		  ckulw_t bsize,
		  int delOver,
		  CkDomain domain,
		  pSOSNode *node);

    virtual ~pSOSPartition();

    int vetoDestroy() {
	return !deleteOverride && getUsedBlks() > 0;
    }

    pSOSNode *getNode() {
	return (pSOSNode *)getNode();
    }

    virtual void protoInit();
};

enum pSOSRegionStates {

    pSOSRegionPENDED =0,
    pSOSRegionAVAIL =1
};

class pSOSRegion : public pSOSSynch {

private:

    CkRegion *region;

    int deleteOverride;

protected:

    virtual void protoInit();

    static const char *stateArray[2];

public:

    pSOSRegion(const char *name,
	       CkExtId uname,
	       void *startAddr,
	       ckulw_t rnSz,
	       ckulw_t unitSz,
	       int delOver,
	       InsertMode mode,
	       pSOSNode *node);

    virtual ~pSOSRegion();

    int vetoDestroy() {
	return !deleteOverride && region->getUsedBytes() > 0;
    }

    ckulw_t getMaxSegSize() const {
	return region->getMaxSegSize();
    }

    ckulw_t getMaxAlloc() const {
	return getMaxSegSize();
    }

    int allocSegment(pSOSTask *task,
		     ckulw_t size,
		     ckticks_t timeout);

    int grabSegment(pSOSTask *task,
		    ckulw_t size);

    int freeSegment(void *seg);

    virtual CkSegmentStatus testSeg(void *chunk);
};

#define pSOS_RNCB(rnsize,unsize) \
    CkAlign2((70 + (rnsize / unsize) * 6),unsize)

#define pSOS_PTCB(ptsize,bufsize) \
    CkAlign2(52 + (((ptsize - 52) / bufsize) / 8),bufsize)

#endif // !_pSOSMemory_h
