/* -*- C++ -*-
 * This file is part of the pSOS simulation model for CarbonKernel.
 * Copyright (C) 2001  Philippe Gerum.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description: Interface to the pSOS+m target abstraction.
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#ifndef _pSOSNode_h
#define _pSOSNode_h

#if defined(__GNUG__) && !defined(__OBSCAN__)
#pragma interface
#endif // __GNUG__ && !__OBSCAN__

#include "CkNode.h"
#include "psos/psos.h"

#ifdef IMPLEMENTING_SO
#undef IMPLEMENTING_SO
#endif // IMPLEMENTING_SO
#define IMPLEMENTING_SO IMPLEMENT_PSOS
#include "sodecl.h"

#define pSOS_MAGIC  0x70736f73	// pSOS magic number for nodeType

// The list of defined index node numbers:
//  inode(n) == Active objects inode number.
// inode(n+1) == Deleted objects inode number.

#define pSOS_TASK_INODE    0
#define pSOS_DTASK_INODE   1
#define pSOS_REGION_INODE  2
#define pSOS_DREGION_INODE 3
#define pSOS_PART_INODE    4
#define pSOS_DPART_INODE   5
#define pSOS_QUEUE_INODE   6
#define pSOS_DQUEUE_INODE  7
#define pSOS_SEMA4_INODE   8
#define pSOS_DSEMA4_INODE  9
#define pSOS_EVTGRP_INODE  10
#define pSOS_DEVTGRP_INODE 11
#define pSOS_TIMER_INODE   12
#define pSOS_DTIMER_INODE  13

#define pSOSRunningNode ((pSOSNode *)CkNode::This)
#define pSOSRunningTask ((pSOSTask *)pSOSRunningNode->getRunningThread())

typedef u_long pSOSArgs[4];

class pSOSRegion;

class pSOSNode : public CkNode {

    friend class pSOSConfig;

protected:

    ckulw_t utask,
	uqueue,
	usema4,
	umsgbuf,
	utimer,
	ulocobj,
	uglbobj,
	uagent;

    ckulw_t nmsgbuf,
	fmsgbuf;

    void (*rootTaskEntry)(...);
    unsigned rootTaskMode;
    unsigned rootTaskPri;

    void (*startCO)(...);
    void (*deleteCO)(...);
    void (*switchCO)(...);

    void (*asyncHandler)(...);
    void (*fatalHandler)(...);

    ckticks_t ticks2slice;

    ckulw_t rn0len,
	rn0usize;

    pSOSRegion *region0;

    CkInterrupt *rtc;

public:

    pSOSNode(const char *nodeName,
	     CkNodeParms *nparms);

    void nameObject(CkObject *namedObject,
		    int type,
		    CkDomain domain,
		    CkExtId uname);

    void unnameObject(CkObject *namedObject);

    CkObject *findActiveObject(CkId id);

    CkObject *findActiveObject(CkExtId uname,
			       int nid,
			       int type);

    int isDeletedObject(CkId id);

    int checkpSOSNodeID(int nid);

    void dispatchAsyncErr(int rc);

    void rtcProgram(double ticksPerSec) {
	rtc->configure(CfEventPeriodical,
		       CString().format("%f msc",1000.0/ticksPerSec));
    }

    ckulw_t holdMsgBufs(ckulw_t count) {

	if (fmsgbuf >= count)
	    {
	    fmsgbuf -= count;
	    return count;
	    }

	return 0;
    } 

    void releaseMsgBufs(ckulw_t count) {
	fmsgbuf += count;
    } 

    void getRPCAgent(CkId pendedID, CkId pendingID) {
	uagent++;
    }

    void freeRPCAgent() {
	uagent--;
    }

    int kdoor(tcreate)(CkExtId uname,
		       u_long pri,
		       unsigned stackSize,
		       ckflags_t flags,
		       CkId *tidp);

    int kdoor(tident)(CkExtId uname,
		      u_long nid,
		      CkId *tidp);

    int kdoor(tstart)(CkId tid,
		      ckflags_t mode,
		      void (*startAddr)(u_long,u_long,u_long,u_long),
		      pSOSArgs *targs);

    int kdoor(trestart)(CkId tid,
			pSOSArgs *targs);

    int kdoor(tdelete)(CkId tid);

    int kdoor(tsuspend)(CkId tid);

    int kdoor(tresume)(CkId tid);

    int kdoor(tsetpri)(CkId tid,
		       u_long newPri,
		       u_long *oldPri);

    int kdoor(tmode)(ckflags_t mask,
		     ckflags_t newMode,
		     ckflags_t *oldMode);

    int kdoor(tgetreg)(CkId tid,
		       u_long regNum,
		       u_long *regValuep);

    int kdoor(tsetreg)(CkId tid,
		       u_long regNum,
		       u_long regValue);

    int kdoor(rncreate)(CkExtId uname,
			void *rnaddr,
			ckulw_t rnSz,
			ckulw_t unitSz,
			ckflags_t flags,
			CkId *rnidp,
			ckulw_t *allocSzp);

    int kdoor(rnident)(CkExtId uname,
		       CkId *rnidp);

    int kdoor(rndelete)(CkId rnid);

    int kdoor(rngetseg)(CkId rnid,
			ckulw_t segSz,
			ckflags_t flags,
			ckticks_t timeout,
			void **segp);

    int kdoor(rnretseg)(CkId rnid,
			void *segp);

    int kdoor(ptcreate)(CkExtId uname,
			void *paddr,
			void *laddr,
			ckulw_t psize,
			ckulw_t bsize,
			ckflags_t flags,
			CkId *ptidp,
			ckulw_t *nbufp);

    int kdoor(ptident)(CkExtId uname,
		       u_long nid,
		       CkId *ptidp);

    int kdoor(ptdelete)(CkId ptid);

    int kdoor(ptgetbuf)(CkId ptid,
			void **bufAddrp);

    int kdoor(ptretbuf)(CkId ptid,
			void *bufAddr);

    int kdoor(qcreate)(CkExtId uname,
		       ckulw_t count,
		       ckflags_t flags,
		       CkId *qidp);

    int kdoor(qvcreate)(CkExtId uname,
			ckflags_t flags,
			ckulw_t count,
			ckulw_t msglen,
			CkId *qidp);

    int kdoor(qdelete)(CkId qid);

    int kdoor(qvdelete)(CkId qid);

    int kdoor(qident)(CkExtId uname,
		      u_long nid,
		      CkId *qidp);

    int kdoor(qvident)(CkExtId uname,
		       u_long nid,
		       CkId *qidp);

    int kdoor(qsend)(CkId qid,
		     void *buf);

    int kdoor(qasend)(CkId qid,
		      void *buf);

    int kdoor(qvsend)(CkId qid,
		      void *buf,
		      ckulw_t length);

    int kdoor(qavsend)(CkId qid,
		       void *buf,
		       ckulw_t length);

    int kdoor(qurgent)(CkId qid,
		       void *buf);

    int kdoor(qaurgent)(CkId qid,
			void *buf);

    int kdoor(qvurgent)(CkId qid,
			void *buf,
			ckulw_t length);

    int kdoor(qavurgent)(CkId qid,
			 void *buf,
			 ckulw_t length);

    int kdoor(qbroadcast)(CkId qid,
			  void *buf,
			  ckulw_t *countp);

    int kdoor(qvbroadcast)(CkId qid,
			   void *buf,
			   ckulw_t length,
			   ckulw_t *countp);

    int kdoor(qreceive)(CkId qid,
			ckflags_t flags,
			ckticks_t timeout,
			u_long buf[4]);

    int kdoor(qvreceive)(CkId qid,
			 ckflags_t flags,
			 ckticks_t timeout,
			 void *buf,
			 ckulw_t length,
			 ckulw_t *msize);

    int kdoor(evsend)(CkId tid,
		      ckflags_t events);

    int kdoor(evasend)(CkId tid,
		       ckflags_t events);

    int kdoor(evreceive)(ckflags_t events,
			 ckflags_t flags,
			 ckticks_t timeout,
			 ckflags_t *eventsp);

    int kdoor(ascatch)(void (*)(cksigmask_t sigs),
		       ckflags_t mode);

    int kdoor(assend)(CkId tid,
		      cksigmask_t sigs);

    int kdoor(asreturn)();

    int kdoor(smcreate)(CkExtId uname,
			ckulw_t count,
			ckflags_t flags,
			CkId *sema4idp);

    int kdoor(smident)(CkExtId uname,
		       u_long nid,
		       CkId *sema4idp);

    int kdoor(smdelete)(CkId sema4id);

    int kdoor(smp)(CkId sema4id,
		   ckflags_t flags,
		   ckticks_t timeout);

    int kdoor(smv)(CkId sema4id);

    int kdoor(smav)(CkId sema4id);

    int kdoor(tmset)(ckulw_t date,
		     ckulw_t time,
		     ckticks_t ticks);

    int kdoor(tmget)(ckulw_t *datep,
		     ckulw_t *timep,
		     ckulw_t *ticksp);

    void kdoor(tmwkafter)(ckticks_t ticks);

    int kdoor(tmwkwhen)(ckulw_t date,
			ckulw_t time,
			ckticks_t ticks);

    int kdoor(tmevafter)(ckticks_t ticks,
			 ckflags_t events,
			 CkId *timeridp);

    int kdoor(tmevwhen)(ckulw_t date,
			ckulw_t time,
			ckticks_t ticks,
			ckflags_t events,
			CkId *timeridp);

    int kdoor(tmevevery)(ckticks_t ticks,
			 ckflags_t events,
			 CkId *timeridp);

    int kdoor(tmcancel)(CkId timerid);

    void kdoor(kfatal)(ckulw_t ecode,
		       ckulw_t flags); 

    int kdoor(kterminate)(u_long nid,
			  ckulw_t fcode,
			  ckulw_t flags); 


    virtual void kinit(coldInit)();

    virtual void kinit(warmInit)();

    virtual ckticks_t rrGetCredit();

    virtual void kcout(fireCallout)(CkEventKind event,
				    CkObject *object);

    virtual void (*getRootEntry())(...);

    virtual void protoInit();

    virtual CkInterrupt *createInterrupt (ckvector_t vector,
					  cklevel_t level,
					  ckisr_t *isr,
					  ckdsr_t *dsr,
					  void *cookie,
					  const char *symbolicName);
};

MakeGList(pSOSNode);

class pSOSInterrupt : public CkInterrupt {

public:

    void (*routine)(void);

    pSOSInterrupt(ckvector_t vector,
		  cklevel_t level,
		  void (*routine)(void),
		  const char *name,
		  pSOSNode *node);
};

#endif // !_pSOSNode_h
