/*
 * This file is part of the pSOS simulation model for CarbonKernel.
 * Copyright (C) 2001  Philippe Gerum.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description: The pSOS+m kernel API this model exports to applications.
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#ifdef __GNUG__
#pragma implementation
#endif // __GNUG__
#include <config.h>
#include "CkManager.h"
#include "pSOSNode.h"

u_long as_catch (void (*asr)(void), u_long mode)

{
    int rc;

    CkInAddr p1("start_addr",(void *)asr);
    CkInFlags p2("mode",mode);

    CkTracePoint t(0,&rc,&p1,&p2,NULL);
    
    rc = pSOSRunningNode->kdoor(ascatch)((void (*)(cksigmask_t))asr,mode);

    t.kdoor(emit)();

    return rc;
}

u_long as_send (u_long tid, u_long sigs)

{
    int rc;

    CkInHDL p1("tid",tid);
    CkInFlags p2("signals",sigs);

    CkTracePoint t(1,&rc,&p1,&p2,NULL);
    
    if (sigs & 0xffff0000)
	CkManager::This->warning("use of reserved signals 0x%.8x",
				 sigs & 0xffff0000);

    rc = pSOSRunningNode->kdoor(assend)(tid,sigs);

    t.kdoor(emit)();

    return rc;
}

u_long as_return (void)

{
    int rc;
    
    CkTracePoint t(2,&rc,NULL);
    
    rc = pSOSRunningNode->kdoor(asreturn)();

    t.kdoor(emit)();

    return rc;
}

u_long ev_asend (u_long tid, u_long events)

{
    int rc;
    
    CkInHDL p1("tid",tid);
    CkInFlags p2("events",events);
    
    CkTracePoint t(3,&rc,&p1,&p2,NULL);
    
    if (events & 0xffff0000)
	CkManager::This->warning("use of reserved event flags 0x%.8x",
				 events & 0xffff0000);

    rc = pSOSRunningNode->kdoor(evasend)(tid,events);

    t.kdoor(emit)();

    return rc;
}

u_long ev_send (u_long tid, u_long events)

{
    int rc;
    
    CkInHDL p1("tid",tid);
    CkInFlags p2("events",events);
    
    CkTracePoint t(4,&rc,&p1,&p2,NULL);
    
    if (events & 0xffff0000)
	CkManager::This->warning("use of reserved event flags 0x%.8x",
				 events & 0xffff0000);

    rc = pSOSRunningNode->kdoor(evsend)(tid,events);

    t.kdoor(emit)();

    return rc;
}

u_long ev_receive (u_long events, u_long flags, u_long timeout, u_long *events_r)

{
    int rc;
    
    CkInFlags p1("events",events);
    CkInFlags p2("flags",flags);
    CkInULong p3("timeout",timeout);
    CkOutFlags p4("events_r",events_r);
    
    CkTracePoint t(5,&rc,&p1,&p2,&p3,&p4,NULL);
    
    if (flags & ~0x3)
	CkManager::This->warning("ev_receive(arg #2): (flags & ~0x3) should be null");

    rc = pSOSRunningNode->kdoor(evreceive)(events,flags,timeout,events_r);

    t.kdoor(emit)();

    return rc;
}

void k_fatal (u_long ecode,
	      u_long flags)
{
    CkInULong p1("error_code",ecode);
    CkInFlags p2("flags",flags);
    
    CkTracePoint t(6,NULL,&p1,&p2,NULL);

    if (ecode < 0x20000000)
	CkManager::This->warning("k_fatal(arg #1): (error_code should be >= 0x20000000");

    pSOSRunningNode->kdoor(kfatal)(ecode,flags);

    t.kdoor(emit)();
}

u_long k_terminate (u_long node,
		    u_long fcode,
		    u_long flags)
{
    int rc;

    CkInULong p1("node",node);
    CkInULong p2("fcode",fcode);
    CkInFlags p3("flags",flags);
    
    CkTracePoint t(7,&rc,&p1,&p2,&p3,NULL);

    if (fcode < 0x20000000)
	CkManager::This->warning("k_terminate(arg #2): (fcode should be >= 0x20000000");

    rc = pSOSRunningNode->kdoor(kterminate)(node,fcode,flags);
    
    t.kdoor(emit)();
    
    return rc;
}

u_long pt_create (char uname[4],
		  void *paddr,
		  void *laddr,
		  u_long length,
		  u_long bsize,
		  u_long flags,
		  u_long *ptid,
		  u_long *nbuf)
{
    int rc;
    
    CkInXID p1("name",uname);
    CkInAddr p2("paddr",paddr);
    CkInAddr p3("laddr",laddr);
    CkInULong p4("length",length);
    CkInULong p5("bsize",bsize);
    CkInFlags p6("flags",flags);
    CkOutHDL p7("ptid",ptid);
    CkOutULong p8("nbuf",nbuf);
	
    CkTracePoint t(8,&rc,&p1,&p2,&p3,&p4,&p5,&p6,&p7,&p8,NULL);
		
    if (flags & ~0x5)
	CkManager::This->warning("pt_create(arg #6): (flags & ~0x5) should be null");

    rc = pSOSRunningNode->kdoor(ptcreate)(uname,
					  paddr,
					  laddr,
					  length,
					  bsize,
					  flags,
					  (CkId *)ptid,
					  nbuf);
    t.kdoor(emit)();

    return rc;
}

u_long pt_delete (u_long ptid)

{
    int rc;
    
    CkInHDL p1("ptid",ptid);
    
    CkTracePoint t(9,&rc,&p1,NULL);
    
    rc = pSOSRunningNode->kdoor(ptdelete)(ptid);

    t.kdoor(emit)();

    return rc;
}

u_long pt_getbuf (u_long ptid, void **bufaddr)

{
    int rc;
    
    CkInHDL p1("ptid",ptid);
    CkOutAddr p2("bufaddr",bufaddr);
    
    CkTracePoint t(10,&rc,&p1,&p2,NULL);
    
    rc = pSOSRunningNode->kdoor(ptgetbuf)(ptid,bufaddr);

    t.kdoor(emit)();

    return rc;
}

u_long pt_ident (char uname[4], u_long nid, u_long *ptid)

{
    int rc;
    
    CkInXID p1("name",uname);
    CkInULong p2("node",nid);
    CkOutHDL p3("ptid",ptid);
    
    CkTracePoint t(11,&rc,&p1,&p2,&p3,NULL);
		
    rc = pSOSRunningNode->kdoor(ptident)(uname,nid,(CkId *)ptid);

    t.kdoor(emit)();

    return rc;
}

u_long pt_retbuf (u_long ptid, void *bufaddr)

{
    int rc;
    
    CkInHDL p1("ptid",ptid);
    CkInAddr p2("bufaddr",bufaddr);
    
    CkTracePoint t(12,&rc,&p1,&p2,NULL);
    
    rc = pSOSRunningNode->kdoor(ptretbuf)(ptid,bufaddr);

    t.kdoor(emit)();

    return rc;
}


u_long q_asend (u_long qid, u_long buf[4])

{
    int rc;
    
    CkInHDL p1("qid",qid);
    CkInAddr p2("msgbuf[]",buf);
    
    CkTracePoint t(13,&rc,&p1,&p2,NULL);

    rc = pSOSRunningNode->kdoor(qasend)(qid,buf);

    t.kdoor(emit)();

    return rc;
}

u_long q_aurgent (u_long qid, u_long buf[4])

{
    int rc;
    
    CkInHDL p1("qid",qid);
    CkInAddr p2("msgbuf[]",buf);
    
    CkTracePoint t(14,&rc,&p1,&p2,NULL);

    rc = pSOSRunningNode->kdoor(qaurgent)(qid,buf);

    t.kdoor(emit)();

    return rc;
}

u_long q_avsend (u_long qid, void *buf, u_long bufsz)

{
    int rc;
    
    CkInHDL p1("qid",qid);
    CkInAddr p2("msgbuf",buf);
    CkInULong p3("msglen",bufsz);
    
    CkTracePoint t(15,&rc,&p1,&p2,&p3,NULL);

    rc = pSOSRunningNode->kdoor(qavsend)(qid,buf,bufsz);

    t.kdoor(emit)();

    return rc;
}

u_long q_avurgent (u_long qid, void *buf, u_long bufsz)

{
    int rc;
    
    CkInHDL p1("qid",qid);
    CkInAddr p2("msgbuf",buf);
    CkInULong p3("msglen",bufsz);
    
    CkTracePoint t(16,&rc,&p1,&p2,&p3,NULL);

    rc = pSOSRunningNode->kdoor(qavurgent)(qid,buf,bufsz);

    t.kdoor(emit)();

    return rc;
}

u_long q_broadcast (u_long qid, u_long buf[4], u_long *count)

{
    int rc;

    CkInHDL p1("qid",qid);
    CkInAddr p2("msgbuf[]",buf);
    CkOutULong p3("count",count);

    CkTracePoint t(17,&rc,&p1,&p2,&p3,NULL);

    rc = pSOSRunningNode->kdoor(qbroadcast)(qid,buf,count);

    t.kdoor(emit)();

    return rc;
}

u_long q_create (char uname[4], u_long count, u_long flags, u_long *qid)

{
    int rc;
    
    CkInXID p1("name",uname);
    CkInULong p2("count",count);
    CkInFlags p3("flags",flags);
    CkOutHDL p4("qid",qid);
    
    CkTracePoint t(18,&rc,&p1,&p2,&p3,&p4,NULL);
    
    if (flags & ~0xf)
	CkManager::This->warning("q_create(arg #3): (flags & ~0xf) should be null");

    rc = pSOSRunningNode->kdoor(qcreate)(uname,count,flags,(CkId *)qid);

    t.kdoor(emit)();

    return rc;
}

u_long q_delete (u_long qid)

{
    int rc;

    CkInHDL p1("qid",qid);

    CkTracePoint t(19,&rc,&p1,NULL);
    
    rc = pSOSRunningNode->kdoor(qdelete)(qid);

    t.kdoor(emit)();

    return rc;
}

u_long q_ident (char uname[4], u_long nid, u_long *qid)

{
    int rc;
    
    CkInXID p1("name",uname);
    CkInULong p2("node",nid);
    CkOutHDL p3("qid",qid);
    
    CkTracePoint t(20,&rc,&p1,&p2,&p3,NULL);
    
    rc = pSOSRunningNode->kdoor(qident)(uname,nid,(CkId *)qid);

    t.kdoor(emit)();

    return rc;
}

u_long q_receive (u_long qid, u_long flags, u_long timeout, u_long buf[4])

{
    int rc;
    
    CkInHDL p1("qid",qid);
    CkInFlags p2("flags",flags);
    CkInULong p3("timeout",timeout);
    CkInAddr p4("msgbuf[]",buf);
    
    CkTracePoint t(21,&rc,&p1,&p2,&p3,&p4,NULL);
    
    if (flags & ~0x1)
	CkManager::This->warning("q_receive(arg #2): (flags & ~0x1) should be null");

    rc = pSOSRunningNode->kdoor(qreceive)(qid,flags,timeout,buf);

    t.kdoor(emit)();

    return rc;
}

u_long q_send (u_long qid, u_long buf[4])

{
    int rc;
    
    CkInHDL p1("qid",qid);
    CkInAddr p2("msgbuf[]",buf);
    
    CkTracePoint t(22,&rc,&p1,&p2,NULL);

    rc = pSOSRunningNode->kdoor(qsend)(qid,buf);

    t.kdoor(emit)();

    return rc;
}

u_long q_urgent (u_long qid, u_long buf[4])

{
    int rc;
    
    CkInHDL p1("qid",qid);
    CkInAddr p2("msgbuf[]",buf);
    
    CkTracePoint t(23,&rc,&p1,&p2,NULL);

    rc = pSOSRunningNode->kdoor(qurgent)(qid,buf);

    t.kdoor(emit)();

    return rc;
}

u_long q_vbroadcast (u_long qid, void *buf, u_long bufsz, u_long *count)

{
    int rc;

    CkInHDL p1("qid",qid);
    CkInAddr p2("msgbuf",buf);
    CkInULong p3("msglen",bufsz);
    CkOutULong p4("count",count);

    CkTracePoint t(24,&rc,&p1,&p2,&p3,&p4,NULL);

    rc = pSOSRunningNode->kdoor(qvbroadcast)(qid,buf,bufsz,count);

    t.kdoor(emit)();

    return rc;
}


u_long q_vcreate (char uname[4],
		  u_long flags,
		  u_long count,
		  u_long msglen,
		  u_long *qid)
{
    int rc;
    
    CkInXID p1("name",uname);
    CkInFlags p2("flags",flags);
    CkInULong p3("maxnum",count);
    CkInULong p4("maxlen",msglen);
    CkOutHDL p5("qid",qid);
    
    CkTracePoint t(25,&rc,&p1,&p2,&p3,&p4,&p5,NULL);
    
    if (flags & ~0xf)
	CkManager::This->warning("q_vcreate(arg #2): (flags & ~0x3) should be null");

    rc = pSOSRunningNode->kdoor(qvcreate)(uname,flags,count,msglen,(CkId *)qid);

    t.kdoor(emit)();

    return rc;
}

u_long q_vdelete (u_long qid)

{
    int rc;

    CkInHDL p1("qid",qid);

    CkTracePoint t(26,&rc,&p1,NULL);
    
    rc = pSOSRunningNode->kdoor(qvdelete)(qid);

    t.kdoor(emit)();

    return rc;
}

u_long q_vident (char uname[4], u_long nid, u_long *qid)

{
    int rc;
    
    CkInXID p1("name",uname);
    CkInULong p2("node",nid);
    CkOutHDL p3("qid",qid);
    
    CkTracePoint t(27,&rc,&p1,&p2,&p3,NULL);

    rc = pSOSRunningNode->kdoor(qvident)(uname,nid,(CkId *)qid);

    t.kdoor(emit)();

    return rc;
}

u_long q_vreceive (u_long qid,
		   u_long flags,
		   u_long timeout,
		   void *buf,
		   u_long bufsz,
		   u_long *msize)
{
    int rc;
    
    CkInHDL p1("qid",qid);
    CkInFlags p2("flags",flags);
    CkInULong p3("timeout",timeout);
    CkInAddr p4("msgbuf",buf);
    CkInULong p5("buflen",bufsz);
    CkOutULong p6("msglen",msize);
    
    CkTracePoint t(28,&rc,&p1,&p2,&p3,&p4,&p5,&p6,NULL);
    
    if (flags & ~0x1)
	CkManager::This->warning("q_vreceive(arg #2): (flags & ~0x1) should be null");

    rc = pSOSRunningNode->kdoor(qvreceive)(qid,flags,timeout,buf,bufsz,msize);

    t.kdoor(emit)();

    return rc;
}

u_long q_vsend (u_long qid, void *buf, u_long bufsz)

{
    int rc;
    
    CkInHDL p1("qid",qid);
    CkInAddr p2("msgbuf",buf);
    CkInULong p3("msglen",bufsz);
    
    CkTracePoint t(29,&rc,&p1,&p2,&p3,NULL);

    rc = pSOSRunningNode->kdoor(qvsend)(qid,buf,bufsz);

    t.kdoor(emit)();

    return rc;
}

u_long q_vurgent (u_long qid, void *buf, u_long bufsz)

{
    int rc;
    
    CkInHDL p1("qid",qid);
    CkInAddr p2("msgbuf",buf);
    CkInULong p3("msglen",bufsz);
    
    CkTracePoint t(30,&rc,&p1,&p2,&p3,NULL);

    rc = pSOSRunningNode->kdoor(qvurgent)(qid,buf,bufsz);

    t.kdoor(emit)();

    return rc;
}

u_long rn_create (char uname[4],
		  void *rnaddr,
		  u_long rnSz,
		  u_long unitSz,
		  u_long flags,
		  u_long *rnidp,
		  u_long *allocSzp)
{
    int rc;

    CkInXID p1("name",uname);
    CkInAddr p2("saddr",rnaddr);
    CkInULong p3("length",rnSz);
    CkInULong p4("unit_size",unitSz);
    CkInFlags p5("flags",flags);
    CkOutHDL p6("rnid",rnidp);
    CkOutULong p7("asiz",allocSzp);
    
    CkTracePoint t(31,&rc,&p1,&p2,&p3,&p4,&p5,&p6,&p7,NULL);

    if (flags & ~0x6)
	CkManager::This->warning("rn_create(arg #5): (flags & ~0x6) should be null");

    rc = pSOSRunningNode->kdoor(rncreate)(uname,
					  rnaddr,
					  rnSz,
					  unitSz,
					  flags,
					  (CkId *)rnidp,
					  allocSzp);
    t.kdoor(emit)();

    return rc;
}

u_long rn_delete (u_long rnid)

{
    int rc;

    CkInHDL p1("rnid",rnid);
    
    CkTracePoint t(32,&rc,&p1,NULL);
		
    rc = pSOSRunningNode->kdoor(rndelete)(rnid);

    t.kdoor(emit)();

    return rc;
}

u_long rn_getseg (u_long rnid,
		  u_long segSz,
		  u_long flags,
		  u_long timeout,
		  void **segp)
{
    int rc;

    CkInHDL p1("rnid",rnid);
    CkInULong p2("size",segSz);
    CkInFlags p3("flags",flags);
    CkInULong p4("timeout",timeout);
    CkOutAddr p5("segaddr",segp);

    CkTracePoint t(33,&rc,&p1,&p2,&p3,&p4,&p5,NULL);
		
    if (flags & ~0x1)
	CkManager::This->warning("rn_getseg(arg #3): (flags & ~0x1) should be null");

    if (segSz == 0)
	return ERR_ZERO;

    rc = pSOSRunningNode->kdoor(rngetseg)(rnid,segSz,flags,timeout,segp);

    t.kdoor(emit)();

    return rc;
}

u_long rn_ident (char uname[4], u_long *rnidp)

{
    int rc;

    CkInXID p1("name",uname);
    CkOutHDL p2("rnid",rnidp);
    
    CkTracePoint t(34,&rc,&p1,&p2,NULL);
		
    rc = pSOSRunningNode->kdoor(rnident)(uname,(CkId *)rnidp);

    t.kdoor(emit)();

    return rc;
}

u_long rn_retseg (u_long rnid, void *segp)

{
    int rc;

    CkInHDL p1("rnid",rnid);
    CkInAddr p2("segaddr",segp);
    
    CkTracePoint t(35,&rc,&p1,&p2,NULL);
    
    rc = pSOSRunningNode->kdoor(rnretseg)(rnid,segp);

    t.kdoor(emit)();

    return rc;
}

u_long sm_av (u_long smid)

{
    int rc;

    CkInHDL p1("smid",smid);

    CkTracePoint t(36,&rc,&p1,NULL);

    rc = pSOSRunningNode->kdoor(smav)(smid);

    t.kdoor(emit)();

    return rc;
}

u_long sm_create (char uname[4], u_long count, u_long flags, u_long *smid)

{
    int rc;

    CkInXID p1("name",uname);
    CkInULong p2("count",count);
    CkInFlags p3("flags",flags);
    CkOutHDL p4("smid",smid);

    CkTracePoint t(37,&rc,&p1,&p2,&p3,&p4,NULL);

    if (flags & ~0x3)
	CkManager::This->warning("sm_create(arg #3): (flags & ~0x3) should be null");

    rc = pSOSRunningNode->kdoor(smcreate)(uname,count,flags,(CkId *)smid);

    t.kdoor(emit)();

    return rc;
}

u_long sm_delete (u_long smid)

{
    int rc;

    CkInHDL p1("smid",smid);

    CkTracePoint t(38,&rc,&p1,NULL);

    rc = pSOSRunningNode->kdoor(smdelete)(smid);

    t.kdoor(emit)();

    return rc;
}

u_long sm_ident (char uname[4], u_long nid, u_long *smid)

{
    int rc;

    CkInXID p1("name",uname);
    CkInULong p2("node",nid);
    CkOutHDL p3("smid",smid);

    CkTracePoint t(39,&rc,&p1,&p2,&p3,NULL);

    rc = pSOSRunningNode->kdoor(smident)(uname,nid,(CkId *)smid);

    t.kdoor(emit)();

    return rc;
}

u_long sm_p (u_long smid, u_long flags, u_long timeout)

{
    int rc;

    CkInHDL p1("smid",smid);
    CkInFlags p2("flags",flags);
    CkInULong p3("timeout",timeout);
    CkTracePoint t(40,&rc,&p1,&p2,&p3,NULL);

    if (flags & ~0x1)
	CkManager::This->warning("sm_p(arg #2): (flags & ~0x1) should be null");

    rc = pSOSRunningNode->kdoor(smp)(smid,flags,timeout);

    t.kdoor(emit)();

    return rc;
}

u_long sm_v (u_long smid)

{
    int rc;

    CkInHDL p1("smid",smid);

    CkTracePoint t(41,&rc,&p1,NULL);

    rc = pSOSRunningNode->kdoor(smv)(smid);

    t.kdoor(emit)();

    return rc;
}

u_long t_create (char uname[4],
		 u_long priority,
		 u_long sstack,
		 u_long ustack,
		 u_long flags,
		 u_long *tid)
{
    int rc;

    CkInXID p1("name",uname);
    CkInULong p2("pri",priority);
    CkInULong p3("sstack",sstack);
    CkInULong p4("ustack",ustack);
    CkInFlags p5("flags",flags);
    CkOutHDL p6("tid",tid);
    
    CkTracePoint t(42,&rc,&p1,&p2,&p3,&p4,&p5,&p6,NULL);

    if (flags & ~0x3)
	CkManager::This->warning("t_create(arg #5): (flags & ~0x3) should be null");

    if (priority < 1 || priority > 255)
	return ERR_PRIOR;

    if (sstack < 128)	// Minimum supervisor stack size is 128 bytes.
	rc = ERR_TINYSTK;
    else
	rc = pSOSRunningNode->kdoor(tcreate)(uname,
					     priority,
					     sstack + ustack,
					     flags,
					     (CkId *)tid);
    t.kdoor(emit)();
    
    return rc;
}

u_long t_delete (u_long tid)

{
    int rc;

    CkInHDL p1("tid",tid);
    
    CkTracePoint t(43,&rc,&p1,NULL);

    rc = pSOSRunningNode->kdoor(tdelete)(tid);

    t.kdoor(emit)();

    return rc;
}

u_long t_getreg (u_long tid, u_long regnum, u_long *regvalp)

{
    int rc;

    CkInHDL p1("tid",tid);
    CkInULong p2("regnum",regnum);
    CkOutULong p3("regvalue",regvalp);
    
    CkTracePoint t(44,&rc,&p1,&p2,&p3,NULL);

    rc = pSOSRunningNode->kdoor(tgetreg)(tid,regnum,regvalp);

    t.kdoor(emit)();

    return rc;
}

u_long t_ident (char uname[4], u_long nid, u_long *tid)

{
    int rc;

    CkInXID p1("name",uname);
    CkInULong p2("node",nid);
    CkOutHDL p3("tid",tid);

    CkTracePoint t(45,&rc,&p1,&p2,&p3,NULL);

    rc = pSOSRunningNode->kdoor(tident)(uname,nid,(CkId *)tid);

    t.kdoor(emit)();
    
    return rc;
}

u_long t_mode (u_long mask, u_long newmode, u_long *oldmode)

{
    int rc;

    CkInFlags p1("mask",mask);
    CkInFlags p2("newmode",newmode);
    CkOutFlags p3("oldmode",oldmode);
    
    CkTracePoint t(46,&rc,&p1,&p2,&p3,NULL);

    rc = pSOSRunningNode->kdoor(tmode)(mask,newmode,oldmode);

    t.kdoor(emit)();

    return rc;
}

u_long t_restart (u_long tid, u_long targs[])

{
    int rc;

    CkInHDL p1("tid",tid);
    CkInAddr p2("targs",targs);
    
    CkTracePoint t(47,&rc,&p1,&p2,NULL);

    rc = pSOSRunningNode->kdoor(trestart)(tid,(pSOSArgs *)targs);

    t.kdoor(emit)();
    
    return rc;
}

u_long t_resume (u_long tid)

{
    int rc;

    CkInHDL p1("tid",tid);
    
    CkTracePoint t(48,&rc,&p1,NULL);

    rc = pSOSRunningNode->kdoor(tresume)(tid);

    t.kdoor(emit)();

    return rc;
}

u_long t_setpri (u_long tid, u_long newpri, u_long *oldpri)

{
    int rc;

    CkInHDL p1("tid",tid);
    CkInULong p2("newprio",newpri);
    CkOutULong p3("oldprio",oldpri);
    
    CkTracePoint t(49,&rc,&p1,&p2,&p3,NULL);

    rc = pSOSRunningNode->kdoor(tsetpri)(tid,newpri,oldpri);

    t.kdoor(emit)();

    return rc;
}

u_long t_setreg (u_long tid, u_long regnum, u_long regval)

{
    int rc;

    CkInHDL p1("tid",tid);
    CkInULong p2("regnum",regnum);
    CkInULong p3("regvalue",regval);

    CkTracePoint t(50,&rc,&p1,&p2,&p3,NULL);

    rc = pSOSRunningNode->kdoor(tsetreg)(tid,regnum,regval);

    t.kdoor(emit)();

    return rc;
}

u_long t_start (u_long tid,
		u_long mode,
		void (*startAddr)(u_long,u_long,u_long,u_long),
		u_long targs[])
{
    int rc;

    CkInHDL p1("tid",tid);
    CkInFlags p2("mode",mode);
    CkInAddr p3("startaddr",(void *)startAddr);
    CkInAddr p4("targs",targs);
    
    CkTracePoint t(51,&rc,&p1,&p2,&p3,&p4,NULL);

    rc = pSOSRunningNode->kdoor(tstart)(tid,mode,startAddr,(pSOSArgs *)targs);

    t.kdoor(emit)();
    
    return rc;
}

u_long t_suspend (u_long tid)

{
    int rc;

    CkInHDL p1("tid",tid);
    
    CkTracePoint t(52,&rc,&p1,NULL);

    rc = pSOSRunningNode->kdoor(tsuspend)(tid);

    t.kdoor(emit)();

    return rc;
}

u_long tm_cancel (u_long tmid)

{
    int rc;

    CkInHDL p1("tmid",tmid);

    CkTracePoint t(53,&rc,&p1,NULL);

    rc = pSOSRunningNode->kdoor(tmcancel)(tmid);

    t.kdoor(emit)();

    return rc;
}

u_long tm_evafter (u_long ticks, u_long events, u_long *tmid)

{
    int rc;

    CkInULong p1("ticks",ticks);
    CkInFlags p2("events",events);
    CkOutHDL p3("tmid",tmid);

    CkTracePoint t(54,&rc,&p1,&p2,&p3,NULL);

    rc = pSOSRunningNode->kdoor(tmevafter)(ticks,events,(CkId *)tmid);
    
    t.kdoor(emit)();

    return rc;
}

u_long tm_evevery (u_long ticks, u_long events, u_long *tmid)

{
    int rc;

    CkInULong p1("ticks",ticks);
    CkInFlags p2("events",events);
    CkOutHDL p3("tmid",tmid);

    CkTracePoint t(55,&rc,&p1,&p2,&p3,NULL);

    rc = pSOSRunningNode->kdoor(tmevevery)(ticks,events,(CkId *)tmid);

    t.kdoor(emit)();

    return rc;
}

u_long tm_evwhen (u_long date, u_long time, u_long ticks, u_long events, u_long *tmid)

{
    int rc;

    CkInULong p1("date",date);
    CkInULong p2("time",time);
    CkInULong p3("ticks",ticks);
    CkInFlags p4("events",events);
    CkOutHDL p5("tmid",tmid);

    CkTracePoint t(56,&rc,&p1,&p2,&p3,&p4,&p5,NULL);

    rc = pSOSRunningNode->kdoor(tmevwhen)(date,time,ticks,events,(CkId *)tmid);
    
    t.kdoor(emit)();

    return rc;
}

u_long tm_get (u_long *date, u_long *time, u_long *ticks)

{
    int rc;

    CkOutULong p1("date",date);
    CkOutULong p2("time",time);
    CkOutULong p3("ticks",ticks);

    CkTracePoint t(57,&rc,&p1,&p2,&p3,NULL);

    rc = pSOSRunningNode->kdoor(tmget)(date,time,ticks);

    t.kdoor(emit)();

    return rc;
}

u_long tm_set (u_long date, u_long time, u_long ticks)

{
    int rc;

    CkInULong p1("date",date);
    CkInULong p2("time",time);
    CkInULong p3("ticks",ticks);

    CkTracePoint t(58,&rc,&p1,&p2,&p3,NULL);

    rc = pSOSRunningNode->kdoor(tmset)(date,time,ticks);

    t.kdoor(emit)();

    return rc;
}

u_long tm_wkafter (u_long ticks)

{
    CkInULong p1("ticks",ticks);

    CkTracePoint t(59,NULL,&p1,NULL);

    pSOSRunningNode->kdoor(tmwkafter)(ticks);

    t.kdoor(emit)();

    return SUCCESS;
}

u_long tm_wkwhen (u_long date, u_long time, u_long ticks)

{
    int rc;

    CkInULong p1("date",date);
    CkInULong p2("time",time);
    CkInULong p3("ticks",ticks);

    CkTracePoint t(60,&rc,&p1,&p2,&p3,NULL);

    rc = pSOSRunningNode->kdoor(tmwkwhen)(date,time,ticks);
    
    t.kdoor(emit)();

    return rc;
}

u_long tm_tick (void)

{
    // Obviosuly no trace point for this one (!)
    pSOSRunningNode->kdoor(tick)();
    return SUCCESS;
}

void i_return (void) {
    // Fake.
}
