/*
 * This file is part of the pSOS simulation model for CarbonKernel.
 * Copyright (C) 2001  Philippe Gerum.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description: Implementation of the pSOS node configuration classes.
 * This code is based on the CfClass package.
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#ifdef __GNUG__
#pragma implementation
#endif // __GNUG__
#include <config.h>
#include "pSOSConfig.h"
#include "pSOSNode.h"
#include "pSOSTask.h"

BEGIN_RINFO
RDEFINE("pSOS+m","pSOS+m target",CKDB_OSNODE,pSOSConfig,CF_NONE,0,9)
END_RINFO

const struct CkInodeType pSOSInodeTypes[] = {

    { "task", pSOS_TASK_INODE },
    { "region", pSOS_REGION_INODE },
    { "partition", pSOS_PART_INODE },
    { "queue", pSOS_QUEUE_INODE },
    { "semaphore", pSOS_SEMA4_INODE },
    { "eventgroup", pSOS_EVTGRP_INODE },
    { "timer", pSOS_TIMER_INODE },
    { NULL, -1 }
};

pSOSConfig::pSOSConfig (const char *_instanceName,
			CfContext *_context) :
    CfClass(_instanceName,_context)
{
    // pSOS+m kernel configuration group
    
    HandlerAttributePointer rootAddrPtr =
	new HandlerAttribute("Root task entry (KC_ROOTSADR)",this,"pSOS+m");
    rootAddrPtr->setValue("root_task");

    IntegerAttributePointer rootPriPtr =
	new IntegerAttribute("Root task priority (KC_ROOTPRI)",this,1,255,"pSOS+m");
    rootPriPtr->setValue(240);

    CheckAttributePointer rootModePtr =
	new CheckAttribute("Root task start mode (KC_ROOTMODE)",this,"pSOS+m");
    rootModePtr->addChoice("Preemption disabled");
    rootModePtr->addChoice("Time slicing enabled");
    rootModePtr->addChoice("ASR enabled");
    rootModePtr->setValue(0x0);

    IntegerAttributePointer rn0LenPtr =
	new IntegerAttribute("Length of region #0 (KC_RN0LEN)",this,1024,2147483647,"pSOS+m");
    rn0LenPtr->setValue(1024);

    IntegerAttributePointer rn0USizePtr =
	new IntegerAttribute("Unit size of region #0 (KC_RN0USIZE)",this,16,2147483647,"pSOS+m");
    rn0USizePtr->setValue(16);

    IntegerAttributePointer nmbufPtr =
	new IntegerAttribute("Size of shared message pool (KC_NMSGBUF)",this,1,65535,"pSOS+m");
    nmbufPtr->setValue(1024);

    IntegerAttributePointer ticks2slicePtr =
	new IntegerAttribute("Round-robin period (KC_TICKS2SLICE)",this,1,2147483647,"pSOS+m");
    ticks2slicePtr->setValue(1);

    HandlerAttributePointer startCoPtr =
	new HandlerAttribute("Task start callout (KC_STARTCO)",this,"pSOS+m");

    HandlerAttributePointer deleteCoPtr =
	new HandlerAttribute("Task delete callout (KC_DELETECO)",this,"pSOS+m");

    HandlerAttributePointer switchCoPtr =
	new HandlerAttribute("Task switch callout (KC_SWITCHCO)",this,"pSOS+m");

    HandlerAttributePointer fatalCoPtr =
	new HandlerAttribute("Fatal error handler (KC_FATAL)",this,"pSOS+m");

    HandlerAttributePointer asyncErrCoPtr =
	new HandlerAttribute("Async error handler (MC_ASYNCERR)",this,"pSOS+m");

    // VRTOS configuration group

    ScaleAttributePointer warpPtr =
	new ScaleAttribute("Target Warp",this,0.0,10.0,0.1,"CarbonKernel");
    warpPtr->setValue(2.0);

    IntegerAttributePointer rtcVectorPtr =
	new IntegerAttribute("RTC interrupt vector",this,0,CK_VECTOR_UMAX,"CarbonKernel");
    rtcVectorPtr->setValue(0);

    IntegerAttributePointer rtcLevelPtr =
	new IntegerAttribute("RTC interrupt level",this,1,7,"CarbonKernel");
    rtcLevelPtr->setValue(1);

    HandlerAttributePointer rtcHandlerPtr =
	new HandlerAttribute("RTC service routine",this,"CarbonKernel");
    rtcHandlerPtr->setValue("tm_tick");

    RealAttributePointer rtcFreqPtr =
	new RealAttribute("RTC Frequency (Hz)",this,1,infinity(),"CarbonKernel");
    rtcFreqPtr->setValue(100.0);

    ResourceAttributePointer interruptPtr =
	new ResourceAttribute("Interrupts",this,CKDB_INTR,"CarbonKernel");

    ResourceAttributePointer panelPtr =
	new ResourceAttribute("Control panels",this,CKDB_PANEL,"CarbonKernel");
}

pSOSConfig::pSOSConfig (DBapl *apl) :
    CfClass(apl) {
}

pSOSConfig::pSOSConfig (pSOSConfig& src) :
    CfClass(src) {
}

void pSOSConfig::instantiate (CfContext *context)

{
    HandlerAttributePointer rootAddrPtr(get("Root task entry (KC_ROOTSADR)"));
    IntegerAttributePointer rootPriPtr(get("Root task priority (KC_ROOTPRI)"));
    CheckAttributePointer rootModePtr(get("Root task start mode (KC_ROOTMODE)"));
    IntegerAttributePointer rn0LenPtr(get("Length of region #0 (KC_RN0LEN)"));
    IntegerAttributePointer rn0USizePtr(get("Unit size of region #0 (KC_RN0USIZE)"));
    IntegerAttributePointer nmbufPtr(get("Size of shared message pool (KC_NMSGBUF)"));
    IntegerAttributePointer ticks2slicePtr(get("Round-robin period (KC_TICKS2SLICE)"));
    HandlerAttributePointer startCoPtr(get("Task start callout (KC_STARTCO)"));
    HandlerAttributePointer deleteCoPtr(get("Task delete callout (KC_DELETECO)"));
    HandlerAttributePointer switchCoPtr(get("Task switch callout (KC_SWITCHCO)"));
    HandlerAttributePointer fatalCoPtr(get("Fatal error handler (KC_FATAL)"));
    HandlerAttributePointer asyncErrCoPtr(get("Async error handler (MC_ASYNCERR)"));

    ScaleAttributePointer warpPtr(get("Target Warp"));
    HandlerAttributePointer rtcVectorPtr(get("RTC interrupt vector"));
    IntegerAttributePointer rtcLevelPtr(get("RTC interrupt level"));
    HandlerAttributePointer rtcHandlerPtr(get("RTC service routine"));
    RealAttributePointer rtcFreqPtr(get("RTC Frequency (Hz)"));
    ResourceAttributePointer interruptPtr(get("Interrupts"));
    ResourceAttributePointer panelPtr(get("Control panels"));

    // Console configuration block
    
    CkConsoleParms cparms;
    cparms.iclistMax = CK_CLIST_INFINITE;
    cparms.oclistMax = CK_CLIST_INFINITE;

    // VRTOS configuration block
    
    CkNodeParms nparms;

    nparms.nodeSerial = getSerialNumber(context);
    nparms.warp = warpPtr->getValue();
    nparms.priMode = PRDN; // i.e. The higher the value, the higher the priority.
    nparms.maxIntrLevel = 7;
    nparms.rtcIntrVector = (ckvector_t)rtcVectorPtr->getValue();
    nparms.rtcIntrLevel = (cklevel_t)rtcLevelPtr->getValue();
    nparms.rtcIntrHandler = rtcHandlerPtr->resolveAddrIfAny(context);
    nparms.rtcHz = rtcFreqPtr->getValue();
    nparms.cparms = cparms;
    nparms.inodeTypes = pSOSInodeTypes;

    // Initialize the pSOS node using current configuration.

    pSOSNode *node = new pSOSNode(getSerialName(context),&nparms);
    node->rootTaskEntry = (void (*)(...))rootAddrPtr->resolveAddr(context);
    node->rootTaskPri = rootPriPtr->getValue();
    node->rootTaskMode = psos2vrtosMode(rootModePtr->getValue());
    node->rn0len = rn0LenPtr->getValue();
    node->rn0usize = rn0USizePtr->getValue();
    node->nmsgbuf = nmbufPtr->getValue();
    node->ticks2slice = ticks2slicePtr->getValue();
    node->startCO = (void (*)(...))startCoPtr->resolveAddrIfAny(context);
    node->switchCO = (void (*)(...))switchCoPtr->resolveAddrIfAny(context);
    node->deleteCO = (void (*)(...))deleteCoPtr->resolveAddrIfAny(context);
    node->fatalHandler = (void (*)(...))fatalCoPtr->resolveAddrIfAny(context);
    node->asyncHandler = (void (*)(...))asyncErrCoPtr->resolveAddrIfAny(context);
    
    CfContextList subContextList;

    if (interruptPtr.valid())
	{
	interruptPtr->instantiate(context,subContextList,node);
	subContextList.destroy();
	}
    
    if (panelPtr.valid())
	{
	panelPtr->instantiate(context,subContextList,node);
	subContextList.destroy();
	}
}
