/*
 * This file is part of the pSOS simulation model for CarbonKernel.
 * Copyright (C) 2001  Philippe Gerum.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description: Specialization of the CarbonKernel trace manager for the
 * pSOS+m kernel's API.
 *
 * Author(s): rpm
 *
 * Contributor(s):
 *
 */

#ifdef __GNUG__
#pragma implementation
#endif // __GNUG__
#include <config.h>
#include "pSOSNode.h"
#include "pSOSProto.h"
#include "pSOSTrace.h"

static const char pSOSTaskFamily[] = "Task";
static const char pSOSRegionFamily[] = "Region";
static const char pSOSPartitionFamily[] = "Partition";
static const char pSOSQueueFamily[] = "Queue";
static const char pSOSEventFamily[] = "Event";
static const char pSOSSignalFamily[] = "Signal";
static const char pSOSSemFamily[] = "Semaphore";
static const char pSOSTimerFamily[] = "Timer";
static const char pSOSKernelFamily[] = "Kernel";

CkTraceDef pSOSTraceInterface::traceDefTable[] = {

    { "as_catch", pSOSSignalFamily }, // 0
    { "as_return", pSOSSignalFamily },
    { "as_send", pSOSSignalFamily },
    { "ev_asend", pSOSEventFamily },
    { "ev_receive", pSOSEventFamily },
    { "ev_send", pSOSEventFamily },
    { "k_fatal", pSOSKernelFamily },
    { "k_terminate", pSOSKernelFamily },
    { "pt_create", pSOSPartitionFamily },
    { "pt_delete", pSOSPartitionFamily },
    { "pt_getbuf", pSOSPartitionFamily }, // 10
    { "pt_ident", pSOSPartitionFamily },
    { "pt_retbuf", pSOSPartitionFamily },
    { "q_asend", pSOSQueueFamily },
    { "q_aurgent", pSOSQueueFamily },
    { "q_avsend", pSOSQueueFamily },
    { "q_avurgent", pSOSQueueFamily },
    { "q_broadcast", pSOSQueueFamily },
    { "q_create", pSOSQueueFamily },
    { "q_delete", pSOSQueueFamily },
    { "q_ident", pSOSQueueFamily }, // 20
    { "q_receive", pSOSQueueFamily },
    { "q_send", pSOSQueueFamily },
    { "q_urgent", pSOSQueueFamily },
    { "q_vbroadcast", pSOSQueueFamily },
    { "q_vcreate", pSOSQueueFamily },
    { "q_vdelete", pSOSQueueFamily },
    { "q_vident", pSOSQueueFamily },
    { "q_vreceive", pSOSQueueFamily },
    { "q_vsend", pSOSQueueFamily },
    { "q_vurgent", pSOSQueueFamily }, // 30
    { "rn_create", pSOSRegionFamily },
    { "rn_delete", pSOSRegionFamily },
    { "rn_getseg", pSOSRegionFamily },
    { "rn_ident", pSOSRegionFamily },
    { "rn_retseg", pSOSRegionFamily },
    { "sm_av", pSOSSemFamily },
    { "sm_create", pSOSSemFamily },
    { "sm_delete", pSOSSemFamily },
    { "sm_ident", pSOSSemFamily },
    { "sm_p", pSOSSemFamily },	// 40
    { "sm_v", pSOSSemFamily },
    { "t_create", pSOSTaskFamily },
    { "t_delete", pSOSTaskFamily },
    { "t_getreg", pSOSTaskFamily },
    { "t_ident", pSOSTaskFamily },
    { "t_mode", pSOSTaskFamily },
    { "t_restart", pSOSTaskFamily },
    { "t_resume", pSOSTaskFamily },
    { "t_setpri", pSOSTaskFamily },
    { "t_setreg", pSOSTaskFamily }, // 50
    { "t_start", pSOSTaskFamily },
    { "t_suspend", pSOSTaskFamily },
    { "tm_cancel", pSOSTimerFamily },
    { "tm_evafter", pSOSTimerFamily },
    { "tm_evevery", pSOSTimerFamily },
    { "tm_evwhen", pSOSTimerFamily },
    { "tm_get", pSOSTimerFamily },
    { "tm_set", pSOSTimerFamily },
    { "tm_wkafter", pSOSTimerFamily },
    { "tm_wkwhen", pSOSTimerFamily } // 60
};

int pSOSTraceInterface::traceDefCount =
sizeof(pSOSTraceInterface::traceDefTable) /
sizeof(CkTraceDef);

CkErrorDef pSOSTraceInterface::errorDefTable[] = {

    { "SUCCESS", 0 },
    { "ERR_TIMEOUT", ERR_TIMEOUT },
    { "ERR_NODENO", ERR_NODENO },
    { "ERR_OBJDEL", ERR_OBJDEL },
    { "ERR_OBJID", ERR_OBJID },
    { "ERR_OBJTYPE", ERR_OBJTYPE },
    { "ERR_OBJNF", ERR_OBJNF },
    { "ERR_TINYSTK", ERR_TINYSTK },
    { "ERR_PRIOR", ERR_PRIOR },
    { "ERR_ACTIVE", ERR_ACTIVE },
    { "ERR_NACTIVE", ERR_NACTIVE },
    { "ERR_SUSP", ERR_SUSP },
    { "ERR_NOTSUSP", ERR_NOTSUSP },
    { "ERR_SETPRI", ERR_SETPRI },
    { "ERR_REGNUM", ERR_REGNUM },
    { "ERR_RNADDR", ERR_RNADDR },
    { "ERR_UNITSIZE", ERR_UNITSIZE },
    { "ERR_TINYUNIT", ERR_TINYUNIT },
    { "ERR_TINYRN", ERR_TINYRN },
    { "ERR_SEGINUSE", ERR_SEGINUSE },
    { "ERR_ZERO", ERR_ZERO },
    { "ERR_TOOBIG", ERR_TOOBIG },
    { "ERR_NOSEG", ERR_NOSEG },
    { "ERR_NOTINRN", ERR_NOTINRN },
    { "ERR_SEGADDR", ERR_SEGADDR },
    { "ERR_SEGFREE", ERR_SEGFREE },
    { "ERR_RNKILLD", ERR_RNKILLD },
    { "ERR_TATRNDEL", ERR_TATRNDEL },
    { "ERR_PTADDR", ERR_PTADDR },
    { "ERR_BUFSIZE", ERR_BUFSIZE },
    { "ERR_TINYPT", ERR_TINYPT },
    { "ERR_BUFINUSE", ERR_BUFINUSE },
    { "ERR_NOBUF", ERR_NOBUF },
    { "ERR_BUFADDR", ERR_BUFADDR },
    { "ERR_BUFFREE", ERR_BUFFREE },
    { "ERR_MSGSIZ", ERR_MSGSIZ },
    { "ERR_BUFSIZ", ERR_BUFSIZ },
    { "ERR_NOMGB", ERR_NOMGB },
    { "ERR_QFULL", ERR_QFULL },
    { "ERR_QKILLD", ERR_QKILLD },
    { "ERR_NOMSG", ERR_NOMSG },
    { "ERR_TATQDEL", ERR_TATQDEL },
    { "ERR_MATQDEL", ERR_MATQDEL },
    { "ERR_VARQ", ERR_VARQ },
    { "ERR_NOTVARQ", ERR_NOTVARQ },
    { "ERR_NOEVS", ERR_NOEVS },
    { "ERR_NOTINASR", ERR_NOTINASR },
    { "ERR_NOASR", ERR_NOASR },
    { "ERR_NOSEM", ERR_NOSEM },
    { "ERR_SKILLD", ERR_SKILLD },
    { "ERR_TATSDEL", ERR_TATSDEL },
    { "ERR_NOTIME", ERR_NOTIME },
    { "ERR_ILLDATE", ERR_ILLDATE },
    { "ERR_ILLTIME", ERR_ILLTIME },
    { "ERR_ILLTICKS", ERR_ILLTICKS },
    { "ERR_BADTMID", ERR_BADTMID },
    { "ERR_TMNOTSET", ERR_TMNOTSET },
    { "ERR_TOOLATE", ERR_TOOLATE },
    { "ERR_ILLRSC", ERR_ILLRSC },
    { "ERR_MASTER", ERR_MASTER }
};

int pSOSTraceInterface::errorDefCount =
sizeof(pSOSTraceInterface::errorDefTable) /
sizeof(CkErrorDef);

pSOSTraceInterface::pSOSTraceInterface (pSOSNode *_node) :
    CkTraceInterface(_node->getOS(),
		     traceDefTable,
		     traceDefCount,
		     errorDefTable,
		     errorDefCount) {
}
