#  This file is part of the pSOS simulation model for CarbonKernel.
#  Copyright (C) 2001  Philippe Gerum.
# 
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  Author(s): rpm
# 
#  Contributor(s):

proc pSOSFlagGroup:attach {context name private} {

    global $context.info
    set $context.info(present) yes
    return [list [list {pSOS kernel} {Event flags} $name] flag]
}

proc pSOSFlagGroup:detach {context} {

    global $context.info
    unset $context.info
    catch { destroy $context }
}

proc pSOSFlagGroup:show {context nid name} {

    if {[winfo exists $context]} {
	wm deiconify $context
	raise $context
    }

    pSOSFlagGroup:pollInfo $context
}

proc pSOSFlagGroup:event {context event} {

    switch $event {
	SimulationHeldEvent {
	    if {[catch { wm state $context } state] == 0} {
		if {$state != "withdrawn"} {
		    pSOSFlagGroup:pollInfo $context
		}
	    }
	}
    }
}

proc pSOSFlagGroup:update {context nid name info} {

    global $context.info

    # dispatch the information block contents
    # to the per-context data array...
    foreach e $info {
	set item [lindex $e 0]
	set value [lindex $e 1]
	set $context.info($item) $value
    }

    set $context.info(current) $info

    if {[winfo exists $context]} {
	set flagfrm [$context.sw1 subwidget frame]

	for {set i 0} {$i < 32} {incr i} {
	    set state [lindex [set $context.info(flags)] $i]
	    if {$state == 1} {
		$flagfrm.$i select
	    } {
		$flagfrm.$i deselect
	    }
	    set col [expr $i % 4]
	    set row [expr $i / 4]
	    grid $flagfrm.$i -column $col -row $row
	}

	set taskfrm [$context.sw2 subwidget frame]
	pSOSTask:fillWaitList $context $taskfrm.list
	return
    }

    # if the flag group object has never been displayed before,
    # build a brand new toplevel for it...

    toplevel $context
    wm title $context "pSOS event flags: $name"
    cascadeWindow $context
    wm protocol $context WM_DELETE_WINDOW \
	"wm withdraw $context"

    # Flags state

    tixLabelFrame $context.sw1 -label Flags \
	-labelside acrosstop -options {
	    label.padx 5
	}

    pack $context.sw1 -expand true -fill both
    set flagfrm [$context.sw1 subwidget frame]

    for {set i 0} {$i < 32} {incr i} {
	set state [lindex [set $context.info(flags)] $i]
	checkbutton $flagfrm.$i -text [format "#%.2d" $i] \
	    -relief flat -anchor w \
	    -variable $context.info(flagstate,$i)
	if {$state == 1} {
	    $flagfrm.$i select
	} {
	    $flagfrm.$i deselect
	}
	set col [expr $i % 4]
	set row [expr $i / 4]
	grid $flagfrm.$i -column $col -row $row
    }

    # Pending tasks

    tixLabelFrame $context.sw2 -label "Pending list" \
	-labelside acrosstop -options {
	    label.padx 5
	}

    pack $context.sw2 -expand true -fill both
    set taskfrm [$context.sw2 subwidget frame]
    tixScrolledListBox $taskfrm.list -scrollbar auto \
	-command "pSOSTask:pickWaitList $context $taskfrm.list"
    pack $taskfrm.list -expand true -fill both
    $taskfrm.list subwidget listbox config -height 5 -bd 2
    pSOSTask:fillWaitList $context $taskfrm.list

    # Button box

    tixButtonBox $context.bbox -orientation horizontal -relief flat -bd 0
    $context.bbox add update -text Update \
	-command "pSOSFlagGroup:pollInfo $context"
    $context.bbox add apply -text Apply \
	-command "pSOSFlagGroup:applyChanges $context"
    $context.bbox add cancel -text Dismiss \
	-command "wm withdraw $context"
    pack $context.bbox -side bottom -fill x
}

proc pSOSFlagGroup:pollInfo {context} {
    TkRequest $context GetObjectInfo all
}

proc pSOSFlagGroup:applyChanges {context} {

    global $context.info

    set oldinfo [set $context.info(current)]
    set flags {}
    for {set i 0} {$i < 32} {incr i} {
	lappend flags [set $context.info(flagstate,$i)]
    }
    set newflags [list flags $flags]

    if {[lsearch -exact $oldinfo $flags] == -1} {
	# send if settings differ only
	TkRequest $context ConfigureObject $flags
    }
}
