#  This file is part of the pSOS simulation model for CarbonKernel.
#  Copyright (C) 2001  Philippe Gerum.
# 
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  Author(s): rpm
# 
#  Contributor(s):

proc pSOSSem:attach {context name private} {

    global $context.info
    set $context.info(present) yes
    return [list [list {pSOS kernel} Semaphores $name] sema4]
}

proc pSOSSem:detach {context} {

    global $context.info
    unset $context.info
    catch { destroy $context }
}

proc pSOSSem:show {context nid name} {

    if {[winfo exists $context]} {
	wm deiconify $context
	raise $context
    }

    pSOSSem:pollInfo $context
}

proc pSOSSem:event {context event} {

    switch $event {
	SimulationHeldEvent {
	    if {[catch { wm state $context } state] == 0} {
		if {$state != "withdrawn"} {
		    pSOSSem:pollInfo $context
		}
	    }
	}
    }
}

proc pSOSSem:update {context nid name info} {

    global $context.info

    # dispatch the information block contents
    # to the per-context data array...
    foreach e $info {
	set item [lindex $e 0]
	set value [lindex $e 1]
	set $context.info($item) $value
    }

    set $context.info(current) $info

    if {[winfo exists $context]} {
	set taskfrm [$context.sw2 subwidget frame]
	pSOSTask:fillWaitList $context $taskfrm.list
	return
    }

    # if the sema4 object has never been displayed before,
    # build a brand new toplevel for it...

    toplevel $context
    wm title $context "pSOS semaphore: $name"
    cascadeWindow $context
    wm protocol $context WM_DELETE_WINDOW \
	"wm withdraw $context"

    # Value

    tixLabelFrame $context.sw1 -labelside none -options {
	label.padx 5
    }

    pack $context.sw1 -expand no -fill both
    set valfrm [$context.sw1 subwidget frame]
    tixControl $valfrm.val \
	-label "Unit count:" \
	-variable $context.info(value) \
	-integer true \
	-min 0 \
	-max 65535 \
	-allowempty false
    $valfrm.val subwidget entry configure -width 5
    pack $valfrm.val -anchor w -pady 5 -padx 5

    # Pending tasks

    tixLabelFrame $context.sw2 -label "Pending list" \
	-labelside acrosstop -options {
	    label.padx 5
	}

    pack $context.sw2 -expand true -fill both
    set taskfrm [$context.sw2 subwidget frame]
    tixScrolledListBox $taskfrm.list -scrollbar auto \
	-command "pSOSTask:pickWaitList $context $taskfrm.list"
    pack $taskfrm.list -expand true -fill both
    $taskfrm.list subwidget listbox config -height 5 -bd 2
    pSOSTask:fillWaitList $context $taskfrm.list

    # Button box

    tixButtonBox $context.bbox -orientation horizontal -relief flat -bd 0
    $context.bbox add update -text Update \
	-command "pSOSSem:pollInfo $context"
    $context.bbox add apply -text Apply \
	-command "pSOSSem:applyChanges $context"
    $context.bbox add cancel -text Dismiss \
	-command "wm withdraw $context"
    pack $context.bbox -side bottom -fill x
}

proc pSOSSem:pollInfo {context} {
    TkRequest $context GetObjectInfo all
}

proc pSOSSem:applyChanges {context} {

    global $context.info

    set oldinfo [set $context.info(current)]
    set newval [list value [set $context.info(value)]]
    
    if {[lsearch -exact $oldinfo $newval] == -1} {
	# sema4 count has changed
	TkRequest $context ConfigureObject $newval
    }
}
